#!/bin/bash
# Usage: $0 dbServerHost dbServerPort userNameForTunnel

# prepare logging
LOGFILE=/tmp/`basename $0 .sh`.log

function log() {
	echo $$: $@ >>$LOGFILE
}

log start $0 at `date` with arguments -"$@"-

# find a free port, assign free port to $port
function getFreePort() {
        while true
        do
                port=$RANDOM
                [ $port -lt 1025 ] && continue
                netstat -an | fgrep -q ':'$port || break
        done
}

# the function does the work (see comments inside), arguments:
#    $1 the remote host
#    $2 the remote port
#    $3 the remote ssh user
function doWork() {
	# get free local port for the ssh tunnel
	getFreePort
	log free port is $port

	# build tunnel to remote host
	log start tunnel to remote host $1 to port localhost:$2 for user $3
	ssh -l $3 -C -L $port:localhost:$2 -N $1 >> $LOGFILE 2>&1 &

	# wait max up to 20 seconds for the tunnel to start
	s=1
	while ! netstat -an | grep -q ':'$port
	do
		log waiting for ssh tunnel -$s-
		sleep 1
		s=$(($s + 1))

		if [ $s -gt 20 ]
		then
			log unable to start ssh tunnel
			exit 1
		fi
	done
	log tunnel built

	# start I/O copying
	log start nc copy program for stdin/stdout and localhost:$port
	nc localhost $port 2>>$LOGFILE

	# kill background tunnel
	log killing ssh tunnel
	kill %1
}

# test arguments: $0 dbServerHost dbServerPort userNameForTunnel
if [ $# -ne 3 ]
then
	log `basename $0` was called with an illegal argument list: "$@"
	exit 1
fi

# do the work
doWork $1 $2 $3

log end of $0 at `date`
exit 0
